#ifndef DSP64_LAPI_H
#define DSP64_LAPI_H

#include "Dsp64Types.h"

#include <std.h>
#include <c64.h>
#include <hwi.h>
#include <tsk.h>
#include <bcache.h>

#include "drv/edma3_drv.h" //lld header

/*******************************************************************************
  MACROS
*******************************************************************************/

#define OTX_DSP_SHADOW_REGION 1

#define DAVINCI_MCBSP_TS_COUNT 128
#define OTXDSP_SDK_FRAMES_PER_BUF  10 //for now, this value is defined here

#define EDMA3_CACHE_LINE_SIZE_IN_BYTES 128 //used for DATA_ALIGN pragma (also used for VICP memory alignment)
#define OTX_ASP_PING_PONG_BUF_SIZE (DAVINCI_MCBSP_TS_COUNT * OTXDSP_SDK_FRAMES_PER_BUF)

#define INTGEN 0x1c40010

#define REG_CLEAR_BIT(num,pos) ((num) &= ~(1u << (pos)))
#define REG_SET_BIT(num,pos)  ((num) |= (1u << (pos)))

#define REG_ADDRESS(base,reg) ((base) + (reg))

#define REG_VALUE(reg) (*((volatile Uint32 *)(reg)))

#define GENERATE_INTARM1 REG_SET_BIT(REG_VALUE(INTGEN),13)

#define CLEAR_INTDSP3 REG_CLEAR_BIT(REG_VALUE(INTGEN),23)

/*DSP inerrupts*/
#define ARM2DSP3  19
#define CCINT1    36
#define CCERRINT  37
#define TCERRINT0 38
#define TCERRINT1 39

#define ARM2DSP3_EVENT_NUM 8
#define CCINT1_EVENT_NUM   9 

#define DAVINCI_DMA_MCBSP1_RX 3
#define DAVINCI_DMA_MCBSP1_TX 2

#define IPR  0x01C01068
#define IPRH 0x01C0106C
#define ICR  0x01C01070
#define ICRH 0x01C01074

#if 0 //we have decided to use the other clearing method (see interrupt handler for details)
#if 0
#define CLEAR_IPR (REG_VALUE(ICR) = (g_bGotDmaRxInterrupt ? IPR_RX_MASK : IPR_TX_MASK))
#else
#define CLEAR_IPR (REG_VALUE(ICR) = (IPR_RX_MASK|IPR_TX_MASK))
#endif
#endif

#if 0 //we don't use this defines, since TCC are changed with lld functions
#define IPR_RX_PING_MASK ((1<<g_sqEdmaCtx[0].nTcc) | (1<<g_sqEdmaCtx[0].nTccPing))
#define IPR_RX_PONG_MASK (1<<g_sqEdmaCtx[0].nTccPong)
#define IPR_RX_MASK (IPR_RX_PING_MASK | IPR_RX_PONG_MASK)

#define IPR_TX_PING_MASK ((1<<g_sqEdmaCtx[1].nTcc) | (1<<g_sqEdmaCtx[1].nTccPing))
#define IPR_TX_PONG_MASK (1<<g_sqEdmaCtx[1].nTccPong)
#define IPR_TX_MASK (IPR_TX_PING_MASK | IPR_TX_PONG_MASK)
#else
//we use master TCC instead
#define IPR_RX_MASK (1<<g_sqEdmaCtx[0].nTcc)
#define IPR_TX_MASK (1<<g_sqEdmaCtx[1].nTcc)
#endif

/**
 * \brief Mapping of DMA channels 0-31 to Hardware Events from
 * various peripherals, which use EDMA for data transfer.
 * All channels need not be mapped, some can be free also.
 * 1: Mapped
 * 0: Not mapped
 *
 * This mapping will be used to allocate DMA channels when user passes
 * EDMA3_DRV_DMA_CHANNEL_ANY as dma channel id (for eg to do memory-to-memory
 * copy). The same mapping is used to allocate the TCC when user passes
 * EDMA3_DRV_TCC_ANY as tcc id (for eg to do memory-to-memory copy).
 *
 * To allocate more DMA channels or TCCs, one has to modify the event mapping.
 */
#define EDMA3_DMA_CHANNEL_TO_EVENT_MAPPING_0          0x3DFF0FFC
/**
 * \brief Mapping of DMA channels 32-63 to Hardware Events from
 * various peripherals, which use EDMA for data transfer.
 * All channels need not be mapped, some can be free also.
 * 1: Mapped
 * 0: Not mapped
 *
 * This mapping will be used to allocate DMA channels when user passes
 * EDMA3_DRV_DMA_CHANNEL_ANY as dma channel id (for eg to do memory-to-memory
 * copy). The same mapping is used to allocate the TCC when user passes
 * EDMA3_DRV_TCC_ANY as tcc id (for eg to do memory-to-memory copy).
 *
 * To allocate more DMA channels or TCCs, one has to modify the event mapping.
 */
#define EDMA3_DMA_CHANNEL_TO_EVENT_MAPPING_1          0x007F1FFF

#define MEMORY_MAP_BASE 0x01C00000
#define TC0_BASE 0x01C10000
#define TC1_BASE 0x01C10400



#define OTX_ASP_BASE	 0x01E02000

#define DRR1	0x00
#define DRR2	0x02
#define DXR1	0x04
#define DXR2	0x06
#define SPCR1	0x08
#define SPCR2	0x0a
#define RCR1	0x0c
#define RCR2	0x0e
#define XCR1	0x10
#define XCR2	0x12
#define SRGR1	0x14
#define SRGR2	0x16
#define MCR1	0x18
#define MCR2	0x1a
#define RCERA	0x1c
#define RCERB	0x1e
#define XCERA	0x20
#define XCERB	0x22
#define PCR0	0x24
#define PCR1	0x26
#define RCERC	0x28
#define RCERD	0x2a
#define XCERC	0x2c
#define XCERD	0x2e
#define RCERE	0x30
#define RCERF	0x32
#define XCERE	0x34
#define XCERF	0x36
#define RCERG	0x38
#define RCERH	0x3a
#define XCERG	0x3c
#define XCERH	0x3e

#define DAVINCI_MAX_MCBSP_COUNT 1

/********************** McBSP SPCR1 bit definitions ***********************/
#define RRST			0x0001
#define RRDY			0x0002
#define RFULL			0x0004
#define RSYNC_ERR		0x0008
#define RINTM(value)	((value)<<4)	/* bits 4:5 */
#define ABIS			0x0040
#define DXENA			0x0080
#define CLKSTP(value)	((value)<<11)	/* bits 11:12 */
#define RJUST(value)	((value)<<13)	/* bits 13:14 */
#define DLB			    0x8000

/********************** McBSP SPCR2 bit definitions ***********************/
#define XRST		0x0001
#define XRDY		0x0002
#define XEMPTY		0x0004
#define XSYNC_ERR	0x0008
#define XINTM(value)	((value)<<4)	/* bits 4:5 */
#define GRST		0x0040
#define FRST		0x0080
#define SOFT		0x0100
#define FREE		0x0200

/********************** McBSP PCR bit definitions *************************/
#define CLKRP		0x0001
#define CLKXP		0x0002
#define FSRP		0x0004
#define FSXP		0x0008
#define DR_STAT		0x0010
#define DX_STAT		0x0020
#define CLKS_STAT	0x0040
#define SCLKME		0x0080
#define CLKRM		0x0100
#define CLKXM		0x0200
#define FSRM		0x0400
#define FSXM		0x0800
#define RIOEN		0x1000
#define XIOEN		0x2000
#define IDLE_EN		0x4000

/********************** McBSP RCR1 bit definitions ************************/
#define RWDLEN1(value)		((value)<<5)	/* Bits 5:7 */
#define RFRLEN1(value)		((value)<<8)	/* Bits 8:14 */

/********************** McBSP XCR1 bit definitions ************************/
#define XWDLEN1(value)		((value)<<5)	/* Bits 5:7 */
#define XFRLEN1(value)		((value)<<8)	/* Bits 8:14 */

/*********************** McBSP RCR2 bit definitions ***********************/
#define RDATDLY(value)		(value)	/* Bits 0:1 */
#define RFIG			0x0004
#define RCOMPAND(value)		((value)<<3)	/* Bits 3:4 */
#define RWDLEN2(value)		((value)<<5)	/* Bits 5:7 */
#define RFRLEN2(value)		((value)<<8)	/* Bits 8:14 */
#define RPHASE			0x8000

/*********************** McBSP XCR2 bit definitions ***********************/
#define XDATDLY(value)		(value)	/* Bits 0:1 */
#define XFIG			0x0004
#define XCOMPAND(value)		((value)<<3)	/* Bits 3:4 */
#define XWDLEN2(value)		((value)<<5)	/* Bits 5:7 */
#define XFRLEN2(value)		((value)<<8)	/* Bits 8:14 */
#define XPHASE			0x8000

/********************* McBSP SRGR1 bit definitions ************************/
#define CLKGDV(value)		(value)	/* Bits 0:7 */
#define FWID(value)		((value)<<8)	/* Bits 8:15 */

/********************* McBSP SRGR2 bit definitions ************************/
#define FPER(value)		(value)	/* Bits 0:11 */
#define FSGM			0x1000
#define CLKSM			0x2000
#define CLKSP			0x4000
#define GSYNC			0x8000

/********************* McBSP MCR1 bit definitions *************************/
#define RMCM			0x0001
#define RCBLK(value)		((value)<<2)	/* Bits 2:4 */
#define RPABLK(value)		((value)<<5)	/* Bits 5:6 */
#define RPBBLK(value)		((value)<<7)	/* Bits 7:8 */

/********************* McBSP MCR2 bit definitions *************************/
#define XMCM(value)		(value)	/* Bits 0:1 */
#define XCBLK(value)		((value)<<2)	/* Bits 2:4 */
#define XPABLK(value)		((value)<<5)	/* Bits 5:6 */
#define XPBBLK(value)		((value)<<7)	/* Bits 7:8 */

#define EDMA_WORD_LENGTH(config) (((config) == OTX_ASP_CONFIG_SLAVE_8MB)? 1: 4)
#define EDMA_FRAME_LENGTH(config) (OTX_ASP_PING_PONG_BUF_SIZE / EDMA_WORD_LENGTH(config))

/*OPT Field Bits (PaRAM Set)*/
#define SAM 0x1 //Src increment mode == 1
#define DAM 0x2 //Dst increment mode == 1
#define SYNCDIM 0x4 //AB-sync == 1
#define STATIC 0x8 //Static PaRAM set == 1
#define FWID(value) ((value)<<8) /*Bits 8:10*/ /*0x6 and 0x7 are reserved values*/
#define TCCMODE (1<<11) //Early completion == 1
#define TCC(value) ((value)<<12) /*Bits 12:17*/
#define TCINTEN (1<<20) //Tc interrupt enabled == 1
#define ITCINTEN (1<<21) //Intermediate tc interrupt enabled == 1
#define TCCHEN (1<<22) //Tc chaining enabled == 1
#define ITCCHEN (1<<23) //Intermediate tc enabled == 1
#define PRIVID(value) ((value)<<24) /*Bits 24:27*/

#define CLEAR_TCC(opt) ((opt) &= 0xFFFC0FFF) //Clear bits 12:17

/*VICP memory macros*/
#define CHIP_MEM(val) (OTX_UINT8 *)(0x11104000 + (val))
#define CHIP_MEM_ALIGNED_SHIFT (((OTX_ASP_PING_PONG_BUF_SIZE / EDMA3_CACHE_LINE_SIZE_IN_BYTES) + 1) * EDMA3_CACHE_LINE_SIZE_IN_BYTES)

/*******************************************************************************
  TYPES
*******************************************************************************/

typedef enum {
  OTX_ASP_CONFIG_MASTER_8MB = 0x0, //Master configuration for ASP with 8MB speed
  OTX_ASP_CONFIG_SLAVE_8MB  = 0x1, //Slave configuration for ASP with 8MB speed
} OtxAspConfigSetE;

typedef struct davinci_mcbsp_reg_cfg {
	OTX_UINT16 spcr2; OTX_UINT16 spcr1;
	OTX_UINT16 rcr2;  OTX_UINT16 rcr1;
	OTX_UINT16 xcr2;  OTX_UINT16 xcr1;
	OTX_UINT16 srgr2; OTX_UINT16 srgr1;
	OTX_UINT16 mcr2;  OTX_UINT16 mcr1;
	OTX_UINT16 pcr2;  OTX_UINT16 pcr0;
	OTX_UINT16 rcerc; OTX_UINT16 rcerd;
	OTX_UINT16 xcerc; OTX_UINT16 xcerd;
	OTX_UINT16 rcere; OTX_UINT16 rcerf;
	OTX_UINT16 xcere; OTX_UINT16 xcerf;
	OTX_UINT16 rcerg; OTX_UINT16 rcerh;
	OTX_UINT16 xcerg; OTX_UINT16 xcerh;
} OtxAspRegConfigS;

typedef struct {
  OTX_UINT8 *pAspReg;//ASP Register (DDR or DXR)
  OTX_UINT8 *pBuf[2];//Ping and Pong buffers (use [0] for Ping and [1] for Pong)
  OTX_UINT32 nChId;//Master logical channel number
  OTX_UINT32 nChIdPing;//Slave logical Ping channel number
  OTX_UINT32 nChIdPong;//Slave logical Pong channel number
  OTX_UINT32 nTcc;//Master tcc
  OTX_UINT32 nTccPing;//Slave tcc for Ping channel
  OTX_UINT32 nTccPong;//Slave tcc for Pong channel
  EDMA3_DRV_PaRAMRegs *pPaRAMSet;//PaRAM set
} OtxEdmaCtxS;

typedef enum {
	DAVINCI_MCBSP_WORD_8 = 0,
	DAVINCI_MCBSP_WORD_12,
	DAVINCI_MCBSP_WORD_16,
	DAVINCI_MCBSP_WORD_20,
	DAVINCI_MCBSP_WORD_24,
	DAVINCI_MCBSP_WORD_32,
} OtxAspWordLength;

/**
 * \brief Semaphore Types
 *
 * Enums for different semaphore types
 */
typedef enum
{
  EDMA3_OS_SEMTYPE_FIFO         = 0,
  EDMA3_OS_SEMTYPE_PRIORITY     = 1
} EDMA3_OS_SemType;


/**
 * \brief Semaphore Attributes Data Structure
 *
 * Data structure for Semaphore Attributes
 */
typedef struct
{
  EDMA3_OS_SemType          type;
  unsigned int              memSegId;
}EDMA3_OS_SemAttrs;

/*******************************************************************************
  GLOBAL VARIABLES
*******************************************************************************/

extern Fxn g_pfIntDsp3Callback; //INTDSP3 interrupt handler callback function
extern short g_bLocalMcAspUsed; //level of serial port usage (0 == not used)
extern volatile short g_bGotDmaRxInterrupt;
extern volatile short g_bGotDmaTxInterrupt;
extern volatile short g_bDmaRxHalf;
extern volatile short g_bDmaTxHalf;

/*--------------------------------------------------------------------------*\
  API FUNCTION DEFINITIONS
\*--------------------------------------------------------------------------*/

void OtxDsp64RegisterIntDsp3(Fxn pfIntDsp3Callback);

short OtxDsp64InitSerialPort(OtxAspConfigSetE eConfig);
short OtxDsp64StartSerialPort(OtxAspConfigSetE eConfig);
short OtxDsp64StopSerialPort(void);
short OtxDsp64CloseSerialPort(void);

#if 1 //for asp/edma work tests
void OtxDsp64ServiceSerialPort(void);
#endif

/******************************************************************************

  Copyright (C) Odin TeleSystems Inc., 2008


  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

******************************************************************************/
#endif
